@@ -36,7 +36,7 @@ from PyQt5.QtWidgets import QLabel
 from qutebrowser.config import config, configexc
 from qutebrowser.keyinput import modeman, modeparsers, basekeyparser
 from qutebrowser.browser import webelem, history
-from qutebrowser.commands import userscripts, runners
+from qutebrowser.commands import runners
 from qutebrowser.api import cmdutils
 from qutebrowser.utils import usertypes, log, qtutils, message, objreg, utils
 if TYPE_CHECKING:
@@ -272,7 +272,7 @@ class HintActions:
         msg = "Yanked URL to {}: {}".format(
             "primary selection" if sel else "clipboard",
             urlstr)
-        message.info(msg, replace=context.rapid)
+        message.info(msg, replace='rapid-hints' if context.rapid else None)
 
     def run_cmd(self, url: QUrl, context: HintContext) -> None:
         """Run the command based on a hint URL."""
@@ -320,16 +320,23 @@ class HintActions:
             elem: The QWebElement to use in the userscript.
             context: The HintContext to use.
         """
+        # lazy import to avoid circular import issues
+        from qutebrowser.commands import userscripts
+
         cmd = context.args[0]
         args = context.args[1:]
+        flags = QUrl.FullyEncoded
+
         env = {
             'QUTE_MODE': 'hints',
             'QUTE_SELECTED_TEXT': str(elem),
             'QUTE_SELECTED_HTML': elem.outer_xml(),
+            'QUTE_CURRENT_URL':
+                context.baseurl.toString(flags),  # type: ignore[arg-type]
         }
+
         url = elem.resolve_url(context.baseurl)
         if url is not None:
-            flags = QUrl.FullyEncoded
             env['QUTE_URL'] = url.toString(flags)  # type: ignore[arg-type]
 
         try:
@@ -677,9 +684,8 @@ class HintManager(QObject):
         Args:
             rapid: Whether to do rapid hinting. With rapid hinting, the hint
                    mode isn't left after a hint is followed, so you can easily
-                   open multiple links. This is only possible with targets
-                   `tab` (with `tabs.background=true`), `tab-bg`,
-                   `window`, `run`, `hover`, `userscript` and `spawn`.
+                   open multiple links. Note this won't work with targets
+                   `tab-fg`, `fill`, `delete` and `right-click`.
             add_history: Whether to add the spawned or yanked link to the
                          browsing history.
             first: Click the first hinted element without prompting.
@@ -747,18 +753,16 @@ class HintManager(QObject):
         if mode_manager.mode == usertypes.KeyMode.hint:
             modeman.leave(self._win_id, usertypes.KeyMode.hint, 're-hinting')
 
-        if rapid:
-            if target in [Target.tab_bg, Target.window, Target.run,
-                          Target.hover, Target.userscript, Target.spawn,
-                          Target.download, Target.normal, Target.current,
-                          Target.yank, Target.yank_primary]:
-                pass
-            elif target == Target.tab and config.val.tabs.background:
-                pass
-            else:
-                name = target.name.replace('_', '-')
-                raise cmdutils.CommandError("Rapid hinting makes no sense "
-                                            "with target {}!".format(name))
+        no_rapid_targets = [
+            Target.tab_fg,  # opens new tab
+            Target.fill,  # exits hint mode
+            Target.right_click,  # opens multiple context menus
+            Target.delete,  # deleting elements shifts them
+        ]
+        if rapid and target in no_rapid_targets:
+            name = target.name.replace('_', '-')
+            raise cmdutils.CommandError(
+                f"Rapid hinting makes no sense with target {name}!")
 
         self._check_args(target, *args)
 
