@@ -1,6 +1,7 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2015-2017 Daniel Schadt
+# Copyright 2015-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2015-2018 Daniel Schadt
 #
 # This file is part of qutebrowser.
 #
@@ -15,7 +16,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Utils for writing an MHTML file."""
 
@@ -25,7 +26,6 @@ import io
 import os
 import re
 import sys
-import collections
 import uuid
 import email.policy
 import email.generator
@@ -33,16 +33,24 @@ import email.encoders
 import email.mime.multipart
 import email.message
 import quopri
+import dataclasses
+from typing import MutableMapping, Set, Tuple, Callable
 
 from PyQt5.QtCore import QUrl
 
 from qutebrowser.browser import downloads
 from qutebrowser.browser.webkit import webkitelem
 from qutebrowser.utils import log, objreg, message, usertypes, utils, urlutils
+from qutebrowser.extensions import interceptors
 
-_File = collections.namedtuple('_File',
-                               ['content', 'content_type', 'content_location',
-                                'transfer_encoding'])
+
+@dataclasses.dataclass
+class _File:
+
+    content: bytes
+    content_type: str
+    content_location: str
+    transfer_encoding: Callable[[email.message.Message], None]
 
 
 _CSS_URL_PATTERNS = [re.compile(x) for x in [
@@ -54,7 +62,7 @@ _CSS_URL_PATTERNS = [re.compile(x) for x in [
 ]]
 
 
-def _get_css_imports_regex(data):
+def _get_css_imports(data):
     """Return all assets that are referenced in the given CSS document.
 
     The returned URLs are relative to the stylesheet's URL.
@@ -71,58 +79,6 @@ def _get_css_imports_regex(data):
     return urls
 
 
-def _get_css_imports_cssutils(data, inline=False):
-    """Return all assets that are referenced in the given CSS document.
-
-    The returned URLs are relative to the stylesheet's URL.
-
-    Args:
-        data: The content of the stylesheet to scan as string.
-        inline: True if the argument is an inline HTML style attribute.
-    """
-    try:
-        import cssutils
-    except (ImportError, re.error):
-        # Catching re.error because cssutils in earlier releases (<= 1.0) is
-        # broken on Python 3.5
-        # See https://bitbucket.org/cthedot/cssutils/issues/52
-        return None
-
-    # We don't care about invalid CSS data, this will only litter the log
-    # output with CSS errors
-    parser = cssutils.CSSParser(loglevel=100,
-                                fetcher=lambda url: (None, ""), validate=False)
-    if not inline:
-        sheet = parser.parseString(data)
-        return list(cssutils.getUrls(sheet))
-    else:
-        urls = []
-        declaration = parser.parseStyle(data)
-        # prop = background, color, margin, ...
-        for prop in declaration:
-            # value = red, 10px, url(foobar), ...
-            for value in prop.propertyValue:
-                if isinstance(value, cssutils.css.URIValue):
-                    if value.uri:
-                        urls.append(value.uri)
-        return urls
-
-
-def _get_css_imports(data, inline=False):
-    """Return all assets that are referenced in the given CSS document.
-
-    The returned URLs are relative to the stylesheet's URL.
-
-    Args:
-        data: The content of the stylesheet to scan as string.
-        inline: True if the argument is an inline HTML style attribute.
-    """
-    imports = _get_css_imports_cssutils(data, inline)
-    if imports is None:
-        imports = _get_css_imports_regex(data)
-    return imports
-
-
 def _check_rel(element):
     """Return true if the element's rel attribute fits our criteria.
 
@@ -174,14 +130,14 @@ class MHTMLWriter:
         root_content: The root content as bytes.
         content_location: The url of the page as str.
         content_type: The MIME-type of the root content as str.
-        _files: Mapping of location->_File namedtuple.
+        _files: Mapping of location->_File object.
     """
 
     def __init__(self, root_content, content_location, content_type):
         self.root_content = root_content
         self.content_location = content_location
         self.content_type = content_type
-        self._files = {}
+        self._files: MutableMapping[QUrl, _File] = {}
 
     def add_file(self, location, content, content_type=None,
                  transfer_encoding=E_QUOPRI):
@@ -236,6 +192,9 @@ class MHTMLWriter:
         return msg
 
 
+_PendingDownloadType = Set[Tuple[QUrl, downloads.AbstractDownloadItem]]
+
+
 class _Downloader:
 
     """A class to download whole websites.
@@ -249,7 +208,6 @@ class _Downloader:
         _finished_file: A flag indicating if the file has already been
                         written.
         _used: A flag indicating if the downloader has already been used.
-        _win_id: The window this downloader belongs to.
     """
 
     def __init__(self, tab, target):
@@ -257,10 +215,9 @@ class _Downloader:
         self.target = target
         self.writer = None
         self.loaded_urls = {tab.url()}
-        self.pending_downloads = set()
+        self.pending_downloads: _PendingDownloadType = set()
         self._finished_file = False
         self._used = False
-        self._win_id = tab.win_id
 
     def run(self):
         """Download and save the page.
@@ -309,9 +266,9 @@ class _Downloader:
         for style in styles:
             style = webkitelem.WebKitElement(style, tab=self.tab)
             # The Mozilla Developer Network says:
-            # type: This attribute defines the styling language as a MIME type
-            # (charset should not be specified). This attribute is optional and
-            # default to text/css if it's missing.
+            # > type: This attribute defines the styling language as a MIME
+            # > type (charset should not be specified). This attribute is
+            # > optional and default to text/css if it's missing.
             # https://developer.mozilla.org/en/docs/Web/HTML/Element/style
             if 'type' in style and style['type'] != 'text/css':
                 continue
@@ -322,7 +279,7 @@ class _Downloader:
         for element in web_frame.findAllElements('[style]'):
             element = webkitelem.WebKitElement(element, tab=self.tab)
             style = element['style']
-            for element_url in _get_css_imports(style, inline=True):
+            for element_url in _get_css_imports(style):
                 self._fetch_url(web_url.resolved(QUrl(element_url)))
 
         # Shortcut if no assets need to be downloaded, otherwise the file would
@@ -338,6 +295,8 @@ class _Downloader:
         Args:
             url: The file to download as QUrl.
         """
+        assert self.writer is not None
+
         if url.scheme() not in ['http', 'https']:
             return
         # Prevent loading an asset twice
@@ -351,8 +310,9 @@ class _Downloader:
         # qute, see the comments/discussion on
         # https://github.com/qutebrowser/qutebrowser/pull/962#discussion_r40256987
         # and https://github.com/qutebrowser/qutebrowser/issues/1053
-        host_blocker = objreg.get('host-blocker')
-        if host_blocker.is_blocked(url):
+        request = interceptors.Request(first_party_url=None, request_url=url)
+        interceptors.run(request)
+        if request.is_blocked:
             log.downloads.debug("Skipping {}, host-blocked".format(url))
             # We still need an empty file in the output, QWebView can be pretty
             # picky about displaying a file correctly when not all assets are
@@ -360,8 +320,7 @@ class _Downloader:
             self.writer.add_file(urlutils.encoded_url(url), b'')
             return
 
-        download_manager = objreg.get('qtnetwork-download-manager',
-                                      scope='window', window=self._win_id)
+        download_manager = objreg.get('qtnetwork-download-manager')
         target = downloads.FileObjDownloadTarget(_NoCloseBytesIO())
         item = download_manager.get(url, target=target,
                                     auto_remove=True)
@@ -377,6 +336,8 @@ class _Downloader:
             url: The original url of the asset as QUrl.
             item: The DownloadItem given by the DownloadManager
         """
+        assert self.writer is not None
+
         self.pending_downloads.remove((url, item))
         mime = item.raw_headers.get(b'Content-Type', b'')
 
@@ -427,6 +388,7 @@ class _Downloader:
             url: The original url of the asset as QUrl.
             item: The DownloadItem given by the DownloadManager.
         """
+        assert self.writer is not None
         try:
             self.pending_downloads.remove((url, item))
         except KeyError:
@@ -457,6 +419,8 @@ class _Downloader:
 
     def _finish_file(self):
         """Save the file to the filename given in __init__."""
+        assert self.writer is not None
+
         if self._finished_file:
             log.downloads.debug("finish_file called twice, ignored!")
             return
@@ -497,8 +461,8 @@ class _Downloader:
         This is needed if a download finishes before attaching its
         finished signal.
         """
-        items = set((url, item) for url, item in self.pending_downloads
-                    if item.done)
+        items = {(url, item) for url, item in self.pending_downloads
+                 if item.done}
         log.downloads.debug("Zombie downloads: {}".format(items))
         for url, item in items:
             self._finished(url, item)
@@ -514,7 +478,6 @@ class _NoCloseBytesIO(io.BytesIO):
 
     def close(self):
         """Do nothing."""
-        pass
 
     def actual_close(self):
         """Close the stream."""
@@ -546,7 +509,7 @@ def start_download_checked(target, tab):
         return
     # The default name is 'page title.mhtml'
     title = tab.title()
-    default_name = utils.sanitize_filename(title + '.mhtml')
+    default_name = utils.sanitize_filename(title + '.mhtml', shorten=True)
 
     # Remove characters which cannot be expressed in the file system encoding
     encoding = sys.getfilesystemencoding()
